/**
 * <p>Title: OWAMP Protocol</p>
 * <p>Description: This section of this project contains the class MySQLcom used to connect to a database and save or read information from it.</p>
 * <p>Copyright: Copyright (c) 2005</p>
 * <p>Company: University of Aveiro / Institute of Telecommunications - Aveiro pole</p>
 * @authors: Helder Veiga and Teresa Pinho
 * @version: 2.1
 * @Last updated: 9-12-2005
 *
 * Copyright (C) 2005  Helder Veiga and Teresa Pinho
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.  A HTML version of
 * the GNU General Public License can be seen at
 * http://www.gnu.org/licenses/gpl.html.
 *
 */

import java.sql.*;  // import needed for JDBC access

public class MySQLcom {

  // Constructors

  public MySQLcom(String serverAddress, String dataBase, String userName, String passWord) throws Exception {
    testDriver ( );  // We should first test if the MySQL driver is installed.
    this.serverAddress = serverAddress;
    this.dataBase = dataBase;
    this.userName = userName;
    this.passWord = passWord;
  }


  // Instance variables

  private String serverAddress;
  private String dataBase;
  private String userName;
  private String passWord;


  // Instance methods

  /**
   * This method is used to test if MySQL JDBC Driver is installed.
   * @throws Exception
   */

  protected void testDriver () throws Exception {
    try {
      Class.forName("com.mysql.jdbc.Driver");
    } catch ( java.lang.ClassNotFoundException e ) {
      System.err.println("Error: MySQL JDBC Driver not installed.");
      throw ( e );
    }
  }


  /**
   * This method is used to connect to a database.
   * @return Connection - the connection to the database is returned.
   * @throws Exception
   */

  protected Connection getConnection () throws Exception {
    String url = "";
    try {
      url = "jdbc:mysql://" + serverAddress + "/" + dataBase;
      Connection con = DriverManager.getConnection(url,userName,passWord);
      return con;
    } catch ( java.sql.SQLException e ) {
      System.err.println("Error: Connection couldn't be established to " + url+".");
      throw ( e );
    }
  }


  /**
   * This method is used to execute a SELECT statement to a database and it returns the result.
   * @param con Connection - connection to the database.
   * @param sqlStatement String - SELECT statement to be executed.
   * @return ResultSet - the result of the execution of the SELECT statement is returned.
   * @throws Exception
   */

  protected ResultSet executeQuery( Connection con, String sqlStatement ) throws Exception {
    try {
      Statement statement = con.createStatement ( );
      ResultSet rs = statement.executeQuery( sqlStatement );
      return rs;
    } catch ( SQLException e ) {
      throw ( e );
    }
  }


  /**
   * This method is used to execute an update statement to the database.
   * @param con Connection
   * @param sqlStatement String - SQL statement to be executed.
   * @throws Exception
   */

  protected void executeUpdate ( Connection con, String sqlStatement ) throws Exception {
    try {
      Statement statement = con.createStatement ( );
      statement.execute ( sqlStatement );
      statement.close ( );
    } catch ( SQLException e ) {
      throw ( e );
    }
  }


  /**
   * This method is used to close the connection to the database.
   * @param con Connection - connection to the database to be closed.
   */

  protected void close(Connection con){
    try{
      con.close();
    }catch(Exception e){
      System.out.println(e);
    }
  }

}
