<?php
  /**
   * Title: processIntervalOfConfidence.php
   * Description: This file contains the code used to process the multiple sessions with confidence intervals
	 * calculation.
   * Copyright: Copyright (c) 2005
   * Company: University of Aveiro / Institute of Telecommunications - Aveiro pole 
   * @author Helder Veiga 
   * @version 2.1
	 * @Last updated: 9-12-2005
 	 *
 	 * Copyright (C) 2005  Helder Veiga 
 	 *
 	 * This program is free software; you can redistribute it and/or
 	 * modify it under the terms of the GNU General Public License
 	 * as published by the Free Software Foundation; either version 2
 	 * of the License, or (at your option) any later version.
 	 *
 	 * This program is distributed in the hope that it will be useful,
 	 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 	 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 	 * GNU General Public License for more details.  A HTML version of 
 	 * the GNU General Public License can be seen at
 	 * http://www.gnu.org/licenses/gpl.html.
 	 *
   */
	 
  // Report all errors except E_NOTICE, E_STRICT & E_WARNING. 
	// We should disable E_WARNING to use fsockopen to detect connectivity to OWAMP elements.
  error_reporting(E_ALL & ~E_NOTICE & ~E_STRICT & ~E_WARNING);
	
  // We must start the session to be able to use the variables $_SESSION, used to define the SID, username
  // and password for the session.
  session_start();

	$link = $_GET['link'];  // Is the link of the page to be presented, after the execution of this request session.
	                        // If this is an EMP web site the JOWAMP_About page should be presented, otherwise the JOWAMP page will be presented.
	if($link!="../jowamp.php")  // EMP web site
	  $_SESSION['EMP'] = 1;
	else // JOWAMP web site
	  $_SESSION['EMP'] = 0;
		
  // Include the function and constant definition file where all needed functions and constants are definied.
	include("./JOWAMP_AuxiliaryFunctions.php");
	include("./JOWAMP_Constants.php");
			
	/* Check if server, sender and Receiver exits. If not, this session will not be started.*/
	
  // Check if server exists
	if(!ConnectionSetup($_SERVER["HTTP_HOST"],$serverTCPport)){
	  $use = $_SERVER["HTTP_HOST"];
    $_SESSION['Error'] = ("Sorry, the machine with IP: <strong>$use</strong> doesn't have a Server (port $serverTCPport) installed or the connection to it is broken.");
		header("Location: JOWAMP_Error.php");
		die();
	}	
		
	// Check if sender exists
  $senderAddress = $_GET['senderAddress'];
	verifySenderAddress(&$senderAddress,&$senderTCPport,&$error);
	if(isset($error)){
	  $_SESSION['Error'] = $error;
		header("Location: JOWAMP_Error.php");
		die();	
	} 
					
	if(!ConnectionSetup($senderAddress,$senderTCPport)){
	  $use = $senderAddress;
    $_SESSION['Error'] = ("Sorry, the machine with IP: <strong>$use</strong> doesn't have a Session-Sender installed (port $senderTCPport) or the connection to it is broken. Please try <a href='JOWAMP_ShowPage.php?link=JOWAMP_MultipleSession.php' title='again'>again</a>.");
		header("Location: JOWAMP_Error.php");
		die();
	}
	
	// Check if receiver exists
	verifyReceiverAddress($_GET['receiverAddress'],&$error);
	if(isset($error)){
	  $_SESSION['Error'] = $error;
		header("Location: JOWAMP_Error.php");
		die();	
	}
	
	if(!ConnectionSetup($_GET['receiverAddress'],$receiverTCPport)){
	  $use = $_GET['receiverAddress'];
    $_SESSION['Error'] = ("Sorry, the machine with IP: <strong>$use</strong> doesn't have a Session-Receiver installed (port $receiverTCPport) or the connection to it is broken. Please try <a href='JOWAMP_ShowPage.php?link=JOWAMP_MultipleSession.php' title='again'>again</a>.");
		header("Location: JOWAMP_Error.php");
		die();
	}	
 
  // When sender or receiver address is loopback address, both have to be loopback address.
	if(((($senderAddress == "127.0.0.1") || ($senderAddress == "localhost"))&&(($_GET['receiverAddress'] != "127.0.0.1") && ($_GET['receiverAddress'] != "localhost"))) ||
	  ((($_GET['receiverAddress'] == "127.0.0.1") || ($_GET['receiverAddress'] == "localhost"))&&(($senderAddress != "127.0.0.1") && ($senderAddress != "localhost")))){
		$_SESSION['Error'] = ("When sender or receiver address is loopback address, both have to be loopback address. Please try <a href='JOWAMP_ShowPage.php?link=JOWAMP_SingleSession.php' title='again'>again</a>.");
		header("Location: JOWAMP_Error.php");
		die(); 
	}
	 
  // Determine mbz_iPVN. Note: Both senderAddress and receiverAddress should be IPv6 or IPv4 addresses. If one of them 
	// is an IPv6 and the other is IPv4 the session should be rejected. As specified in the OWAMP specification, they should
	// have the same IP version. 
	if(substr_count($_GET['senderAddress'], "[")&&substr_count($_GET['receiverAddress'], "[")) // IPv6
	  $mbz_iPVN = 6;
	else  // Not IPv6
	  if(!substr_count($_GET['senderAddress'], "[")&&!substr_count($_GET['receiverAddress'], "[")) // IPv4
		  $mbz_iPVN = 4;
		else{
		  $_SESSION['Error'] = ("Sorry, IP addresses of sender and receiver should have the same version (IPv6 or IPv4). Please try <a href='JOWAMP_ShowPage.php?link=JOWAMP_MultipleSession.php' title='again'>again</a>.");
		  header("Location: JOWAMP_Error.php");
		  die();
	  }
		
	$confSender = 0;
	$confReceiver = 1;
	$numberOfScheduleSlots = $_GET['numberOfScheduleSlots'];
	$numberOfPackets = $_GET['numberOfPackets'];
	$senderPort = 0;  // use a free port
	$receiverPort = 0;  // use a free port
	$senderAddress = $senderAddress.":$senderTCPport";
	$receiverAddress = $_GET['receiverAddress'];
	$paddingLength = $_GET['paddingLength'];
	//StartTime
	$startYear = $_GET['startYear'];
	$startMonth = $_GET['startMonth'];
	$startDate = $_GET['startDate'];
	$startHour = $_GET['startHour'];
	$startMinute = $_GET['startMinute'];
	$startSecond = $_GET['startSecond'];
	$timeOut = $_GET['timeout'];  // timeOut in seconds
	$typePDescriptor_Type = 0;
	$typePDescriptor_Value = 0;
	$slotType = $_GET['slotType'];
	$slotParameter = $_GET['slotParameter'];  // slotParameter in seconds
	if($slotParameter==0)  // Can not be zero.
	  $slotParameter = 1;
	//StopTime
	$stopYear = $_GET['stopYear'];
	$stopMonth = $_GET['stopMonth'];
	$stopDate = $_GET['stopDate'];
	$stopHour = $_GET['stopHour'];
	$stopMinute = $_GET['stopMinute'];
	$stopSecond = $_GET['stopSecond'];
	$timeBetweenSetOfTests = $_GET['timeBetweenSetOfTests'];  // in minutes
	$numOfTestsInInterval = $_GET['numOfTestsInInterval'];       
	$timeBetweenTests = $_GET['timeBetweenTests'];            // in seconds
 	
	mysql_connect($DB_HOSTNAME, $DB_USERNAME, $DB_PASSWORD) or die('Problem in the connection to MySQL Server: '. mysql_error());

  // Save Confidence Interval Sessions configuration to table jowamp_CI. If It doesn't exist it 
	// should be created.
   
  // Test if startTime is not in the past. If startTime in the past the session should not be 
	// processed.
                  
	$startTime = mktime ( $startHour, $startMinute, $startSecond, $startMonth, $startDate, $startYear);
  $stopTime = mktime ( $stopHour, $stopMinute, $stopSecond, $stopMonth, $stopDate, $stopYear); 
  
	if($startTime < getTime()){
	  $_SESSION['Error'] = "The Start time introduced is not valid (The Start time is in the past). Please try <a href='JOWAMP_ShowPage.php?link=JOWAMP_MultipleSession.php' title='again'>again</a>.";
	  header("Location: JOWAMP_Error.php");
		die();
	}
	
	// Determine the number of intervals of test between start and end of tests.
	
	$numIntervals = floor(($stopTime-$startTime)/(($timeBetweenSetOfTests*60)));
              
	if($numIntervals < 0){
	  $_SESSION['Error'] = "The Stop time introduced is not valid (Stop time < Start time). Please try <a href='JOWAMP_ShowPage.php?link=JOWAMP_MultipleSession.php' title='again'>again</a>.";
	  header("Location: JOWAMP_Error.php");
		die();
  }

	if($numIntervals == 0){
	  $_SESSION['Error'] = "The total test duration is longer than the interarrival time between sets of tests. Please try <a href='JOWAMP_ShowPage.php?link=JOWAMP_MultipleSession.php' title='again'>again</a>.";
	  header("Location: JOWAMP_Error.php");
		die();
  }
  
	// Verify total tests duration and interval duration
	
	if($slotType == 1)  // the parameter is the delay itself
	  $parameterTime = $slotParameter;
	else                // exponentially distributed pseudo-random quantity - max value = lambda = 1/mean
		$parameterTime = 1/$slotParameter;
		
	$totalTimeToWait = ($numberOfPackets-1)*$parameterTime + $timeOut; // in seconds
	
	if(($numOfTestsInInterval*$timeBetweenTests > ($timeBetweenSetOfTests*60))||(($timeBetweenTests*($numOfTestsInInterval-1)+$totalTimeToWait) > ($timeBetweenSetOfTests*60))){
    $_SESSION['Error'] = "The total tests duration is longer than interval between set of tests. ".
    "Possible errors:<br><br>".            
    "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;i)&nbsp;&nbsp;&nbsp;The number of tests times test duration is longer than interval between set of tests.<br>".
    "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;ii)&nbsp;&nbsp;The duration of a set of sessions is longer than the interval between set of tests.<br><br>".
		"&nbsp;&nbsp;&nbsp;&nbsp;Please try <a href='JOWAMP_ShowPage.php?link=JOWAMP_MultipleSession.php' title='again'>again</a>.";
	  header("Location: JOWAMP_Error.php");
		die();           
  }

	if((($numberOfPackets-1)*$parameterTime + $timeOut) > $timeBetweenTests){
	  $_SESSION['Error'] = "The test duration is longer than the time interval between tests. ".
    "Possible errors:<br><br>".            
    "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;i)&nbsp;&nbsp;&nbsp;Number of packets to big.<br>".
    "&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;ii)&nbsp;&nbsp;Parameter time too long.<br>".
		"&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;iii) Time out time too long.<br><br>".
		"&nbsp;&nbsp;&nbsp;&nbsp;Please try <a href='JOWAMP_ShowPage.php?link=JOWAMP_MultipleSession.php' title='again'>again</a>.";
	  header("Location: JOWAMP_Error.php");
		die();           
  }
	
	// Verify if the TableCI exists. If not it should be created.
	 	
  $sql = " SELECT MAX(id) FROM $TableCI";
  $resultado = mysql_db_query($DB_NAME, $sql);
  if(!$resultado){
    $sql = " CREATE TABLE $TableCI (id INT NOT NULL AUTO_INCREMENT, userid VARCHAR(39), mbzIPVN VARCHAR(1), confSender BIT, confReceiver BIT, numberOfScheduleSlots INT, numberOfPackets INT, senderPort INT, receiverPort INT, senderAddress VARCHAR(47), receiverAddress VARCHAR(41), paddingLength INT, startHour INT(2), startMinute INT(2), startSecond INT(2), startDate INT(2), startMonth INT(2), startYear INT(4), timeOut DOUBLE, typePDescriptor_Type VARCHAR(4), typePDescriptor_Value VARCHAR(4), slotType INT(1), slotParameter DOUBLE, stopHour INT(2), stopMinute INT(2), stopSecond INT(2), stopDate INT(2), stopMonth INT(2), stopYear INT(4), timeBetweenSetOfTests DOUBLE, timeBetweenTests DOUBLE, numIntervals INT, numOfTestsInInterval INT, PRIMARY KEY (id))";
		mysql_db_query($DB_NAME, $sql) or die('Could not create table: ' . mysql_error());
	}
 
	// Now we should insert the Confidence Interval session configurations in the TableCI and get the row id to be used
	// to name the CIconfigTable (table with the initial configurations of a confidence interval session) and CIsidTable 
	// (table with the session's SID, senderPort, receiverPort, real start time and total test duration for each  
	// individual single session of each interval of a confidence interval session). This id should be unique for each 
	// session started, so we can use it to identify each CI session configured.
	// Using LOCK Tables is an effective way to retrieve the last insert id without fail or having two queries inserted 
	// at the same time causing an unexpected number being returned.
	// Note: The OWAMP_ControlClient will also create an table (CIconfigTable) where all initial configuration of a CI 
	// session are saved, but to guanrantee the access to each configured session we should use a table (TableCI) where 
	// this configuration is saved for each CI session started on this site. This table guarantees that on each unique 
	// row there are the initial configurations of an CI session. This table will be used to list all the CI sessions
	// configured on this site (Results section). 

	mysql_db_query($DB_NAME,"LOCK TABLES apc_forms WRITE");
	mysql_db_query($DB_NAME,"SET AUTOCOMMIT = 0");
	$sql = "INSERT INTO $TableCI (userid, mbzIPVN, confSender, confReceiver, numberOfScheduleSlots, numberOfPackets, senderPort, receiverPort, senderAddress, receiverAddress, paddingLength, startHour, startMinute, startSecond, startDate, startMonth, startYear, timeOut, typePDescriptor_Type, typePDescriptor_Value, slotType, slotParameter, stopHour, stopMinute, stopSecond, stopDate, stopMonth, stopYear, timeBetweenSetOfTests, timeBetweenTests, numIntervals, numOfTestsInInterval) VALUES ('$userID', '$mbz_iPVN', $confSender, $confReceiver, $numberOfScheduleSlots, $numberOfPackets, $senderPort, $receiverPort, '$senderAddress', '$receiverAddress', $paddingLength, $startHour, $startMinute, $startSecond, $startDate, $startMonth, $startYear, $timeOut, '$typePDescriptor_Type', '$typePDescriptor_Value', $slotType, $slotParameter, $stopHour, $stopMinute, $stopSecond, $stopDate, $stopMonth, $stopYear, $timeBetweenSetOfTests, $timeBetweenTests, $numIntervals, $numOfTestsInInterval)";
  mysql_db_query($DB_NAME, $sql) or die('Could not insert configurations into table: ' . mysql_error());
	$resultado = mysql_db_query($DB_NAME,"SELECT LAST_INSERT_ID()");
	$registo=mysql_fetch_array($resultado);
	$CI = $registo[0];  // The row id of this new entry.
	mysql_db_query($DB_NAME,"COMMIT");
	mysql_db_query($DB_NAME,"UNLOCK TABLES");

	$CIconfigTable = "configTable_CI_".$CI;
	$CIsidTable = "sidTable_CI_".$CI;
 
	mysql_close();
											
  if(strlen($NTPserverAddress) == 0)											
	  $parameters = "$userID $serverAddress $confSender $confReceiver $numberOfScheduleSlots $numberOfPackets $senderPort $receiverPort $senderAddress $receiverAddress $paddingLength $startYear $startMonth $startDate $startHour $startMinute $startSecond $timeOut $typePDescriptor_Type $typePDescriptor_Value $slotType $slotParameter $DB_HOSTNAME $DB_NAME $DB_USERNAME $DB_PASSWORD $CIconfigTable $CIsidTable $timeBetweenSetOfTests $timeBetweenTests $numIntervals $numOfTestsInInterval";
  else
  	$parameters = "$userID $serverAddress $confSender $confReceiver $numberOfScheduleSlots $numberOfPackets $senderPort $receiverPort $senderAddress $receiverAddress $paddingLength $startYear $startMonth $startDate $startHour $startMinute $startSecond $timeOut $typePDescriptor_Type $typePDescriptor_Value $slotType $slotParameter $DB_HOSTNAME $DB_NAME $DB_USERNAME $DB_PASSWORD $CIconfigTable $CIsidTable $timeBetweenSetOfTests $timeBetweenTests $numIntervals $numOfTestsInInterval $NTPserverAddress $updateInterval";
 
  execInBackground("java -classpath ./owamp_protocol OWAMP_ControlClient","$parameters");
	
?>
 
<meta HTTP-EQUIV="REFRESH" content="0; url=<?php echo $link; ?>">
 
